function Initialize()
end

function GenerateHOC()
  local v = { }
  
  local match  = string.match
  local gmatch = string.gmatch
  local max    = math.max
  local min    = math.min

  local lRoot  = SKIN:GetVariable('ROOTCONFIGPATH')

  -- load skinframework
  dofile(lRoot..'SkinFramework.lua')

  varFile = SkinFile.new('#@#Variables.inc')
  v = varFile:readSection('Variables')

  local lCommon   = lRoot..v.Config:gsub(" ","")..'.ini'
  local lBands   = lRoot..'bands.inc'
  
  local function SetKV(type,sec,key,val,dst)
    SKIN:Bang(type,sec,key,val,dst)
  end
  local function SetFileKV(s,k,v,p)    SetKV('!WriteKeyValue',s,k,v,p)         end
  local function SetLiveKV(s,k,d)      SetKV('!SetOption',s,k,d,v.Config)      end
  local function SetLiveGroupKV(g,k,d) SetKV('!SetOptionGroup',g,k,d,v.Config) end
  local function clamp(n,min,max)      return math.min(max,math.max(min,n))    end

  local halfwidth = (v.Radius + v.BarHeight) * v.Scale

  -- load skinframework
  dofile(lRoot..'SkinFramework.lua')

  bandsFile = SkinFile.new(lBands)
  bandsFile:beginGen()

  
  --###########################
  --# GENERATE AUDIO MEASURES #
  --###########################
  local bands = v.Bands-1 --zero based
  local audioMeasures = 0

  -- if we mirror the vis, then we only need half as many audio measures
  if v.Mirror == "1" then
    if (v.Bands % 2 == 0) then
      audioMeasures = bands-(v.Bands/2)
    else
      audioMeasures = (bands/2)
    end
  else
    audioMeasures = bands
  end

  print("audioMeasures: "..audioMeasures)


  for i = 0,audioMeasures,1
  do
    local kvBarAudio = {
      Plugin               = 'AudioLevelBeta',
      Parent               = 'Audio',
      Type                 = 'Band',
      Group                = 'Audio',
      AverageSize          = v.AveragingPastValuesAmount,
      BandIdx              = i
    }
    bandsFile:genMeasure('Audio'..i, 'Plugin', kvBarAudio)
  end

  local bandcolors = v.Gradient == "1" and Gradient.Parse(v.GColor):GetValues(v.Bands) or 0

  --###################
  --#  GENERATE BARS  #
  --###################

  for i = 0,bands,1
  do

    local angle
    local a

    if v.Mirror == "1" then
      if v.InvertMirror == '1' then
        angle = v.StartAngle+((v.EndAngle/v.Bands + v.AngularDisplacement) * (i+0.5))
        a = i <= audioMeasures and i or i-audioMeasures-1
      else
        angle = v.StartAngle+((v.EndAngle/v.Bands + v.AngularDisplacement) * (i+0.5))
        a = i <= audioMeasures and i or bands-i
      end
    else
      angle = v.StartAngle+((v.EndAngle/v.Bands + v.AngularDisplacement) * (i+0.5))
      a = i
    end

    local tmatrix = TransformationMatrix.Create(halfwidth,halfwidth)
    tmatrix:Rotate(angle)
    tmatrix:Scale(v.Scale)

    local measureName = 'Audio'..a

    local kvBar = {
      AntiAlias            = 1,
      TransformationMatrix = tmatrix:ToString(),
      X                    = halfwidth - v.BarWidth/2,
      Y                    = 0,
      W                    = v.BarWidth,
      H                    = v.BarHeight,
      BarOrientation       = 'Vertical',
      Group                = 'Bars',
      BarColor             = v.Gradient == "1" and bandcolors[i]:ToString() or v.SColor,
      Flip                 = v.InvertBars
    }
    bandsFile:genMeter(i, 'Bar', measureName, kvBar)

    --SKIN:Bang('!UpdateMeter',  i-1,         v.Config)
    --SKIN:Bang('!UpdateMeasure','Audio'..i-1,v.Config)
  end

  bandsFile:endGen()

  --#############################
  --# GEN. CENTRAL AUDIO PARENT #
  --#############################

  local smoothingMode = 1
  if v.Mirror == "1" then
    if v.InvertMirror == '1' then
      smoothingMode = 1
    else
      smoothingMode = 2
    end
  end

  local kvAudio = {
    Bands            = audioMeasures + 1,
    FFTSize          = v.FFTSize,
    FFTBufferSize    = v.FFTBufferSize,
    FFTAttack        = v.FFTAttack,
    FFTDecay         = v.FFTDecay,
    FreqMin          = v.FreqMin,
    FreqMax          = v.FreqMax,
    Sensitivity      = v.Sensitivity,
    Smoothing        = v.Smoothing,
    SmoothingMode    = smoothingMode,
    UpdatesPerSecond = v.FPS
  }
  --for key,val in pairs(kvAudio) do SetLiveKV('Audio', key, val) end
  for key,val in pairs(kvAudio) do SetFileKV('Audio', key, val, lCommon) end

  --######################
  --# SET BACKGROUND W/H #
  --######################

  SetFileKV('MeterBG', 'W', halfwidth*2, lCommon)
  SetFileKV('MeterBG', 'H', halfwidth*2, lCommon)

  SKIN:Bang('!Refresh')

  --print("BEAT CIRCLE LOADED")
end